
// code by ido roseman

#include <Servo.h>

// light sensitive resistors are connected with one pin to ground, and the second with 22K resistor to VCC
const int led_pins[] = { 3, 1, 7, 5 };
const int servo_pins[] = { 11, 10, 9, 6 };
const int ldr_pins[] = {A0, A1, A2, A3 };
const char names[] = ".RYBG";

Servo servo[4]; 
byte sequence[32];
byte prev_sequence[32];
byte indx = 0;
byte prev_indx = 0;
const int thresh = 400;

const bool talkative = true;
const bool graph = !talkative;

//--------------------------------------------------------------------

void setup() {
  Serial.begin(9600);
  while (!Serial)
    ;
  if (talkative) {
    Serial.println("Simon Player!");
    Serial.println("-------------");
  }
  for (int i=0; i<4; i++)
    pinMode(led_pins[i], OUTPUT);
  for (int i=0; i<4; i++){
    servo[i].attach(servo_pins[i]);
    servo[i].write(110);
  }
}

//--------------------------------------------------------------------

byte scan(){
  int pressed = 0;
  int count = 0;
  
  int n = 4;
  int acc[4] = { 0, 0, 0, 0 };
  
  // wait for light + timeout
  while (pressed == 0) {
    // avarage n readings
    for (int i=0; i<4; i++)
      acc[i] = 0;
    for (int j=0; j<n; j++)
      for (int i=0; i<4; i++) 
        acc[i] += analogRead(ldr_pins[i]);
    for (int i=0; i<4; i++) {
      if (acc[i] < thresh * n)
        pressed = i+1;
    }
    if (graph){
      for (int i=0; i<4; i++) {
        Serial.print(acc[i], DEC);
        Serial.print(' ');
      }
      Serial.println();
    }
    delay(10);
    if (count++ > 100)
      return 0;
  }

  // wait for light to end
  count = 0;
  int val = acc[pressed-1] ;
  digitalWrite(led_pins[pressed-1], HIGH); 
  while (val < thresh * n) {
    val = 0;
    for (int j=0; j<n ; j++)
      val += analogRead(ldr_pins[pressed-1]);
    if (graph){
      for (int i=0; i<4; i++){
        Serial.print(i == pressed-1 ? val : 0, DEC);
        Serial.print(' ');
      }
      Serial.println();
    }
      
    count ++;
    delay(10);
  }
  digitalWrite(led_pins[pressed-1], LOW); 
  return pressed;
}

//--------------------------------------------------------------------

void press(byte i){
    int count = 0;
    delay(150);
    digitalWrite(led_pins[i-1], HIGH); 
    servo[i-1].write(180);
    // wait for light 
    while ((analogRead(ldr_pins[i-1]) > 500) && (count < 500)) {
      count++;
      delay(1);
    }
    servo[i-1].write(110);
    delay(200);
    digitalWrite(led_pins[i-1], LOW);
}

//--------------------------------------------------------------------

void loop() {
    byte val;
    bool needToPlayback = false;

    // record
    if (talkative)
      Serial.print("in :");
    while ((val = scan()) > 0){
      if (talkative)
        Serial.print(names[val]);
      sequence[indx++] = val;
      needToPlayback = true;
    }
    if (talkative)
      Serial.println();

    // check for victory
    if ((prev_indx > 4) && (indx == 4)) {
      bool victory = true;
      for (int i=0; i<4; i++)
        victory &= sequence[i] == prev_sequence[prev_indx-1];
      if (victory && talkative) {
        Serial.println("victory"); 
        // dance
          for (int i=0; i<3; i++) {
            servo[0].write(50); delay(200);
            servo[2].write(50); delay(200);
            servo[1].write(50); delay(200);
            servo[3].write(50); delay(200);
            servo[0].write(110); delay(200);
            servo[2].write(110); delay(200);
            servo[1].write(110); delay(200);
            servo[3].write(110); delay(200);
            }
        indx = 0;
      }
    }
    for (int i=0; i<indx; i++)
      prev_sequence[i] = sequence[i];
    prev_indx = indx;
      
    if (needToPlayback){
      // playback
      if (talkative)
        Serial.write("out:");
      for (int i=0; i<indx; i++) {
        press(sequence[i]);
        if (talkative)
          Serial.print(names[sequence[i]]);
      }
      if (talkative)
        Serial.println();
      // pause
      delay(500);
      needToPlayback = false;
      indx = 0;
    }
    else {
      // game ended
      if (indx>0) {
        if (talkative)
          Serial.println("reset");
      }
      indx = 0;
    }

}
